package cn.iocoder.yudao.module.dms.service.articlecatogry;

import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.dms.controller.admin.articlecatogry.vo.ArticleCatogryListReqVO;
import cn.iocoder.yudao.module.dms.controller.admin.articlecatogry.vo.ArticleCatogrySaveReqVO;
import cn.iocoder.yudao.module.dms.dal.dataobject.articlecatogry.ArticleCatogryDO;
import cn.iocoder.yudao.module.dms.dal.pgsql.articlecatogry.ArticleCatogryMapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.dms.enums.ErrorCodeConstants.*;

/**
 * 文章分类 Service 实现类
 *
 * @author dzx
 */
@Service
@Validated
public class ArticleCatogryServiceImpl implements ArticleCatogryService {

    @Resource
    private ArticleCatogryMapper articleCatogryMapper;

    @Override
    public Long createArticleCatogry(ArticleCatogrySaveReqVO createReqVO) {
        // 校验父ID的有效性
        validateParentArticleCatogry(null, createReqVO.getParentId());
        // 校验父分类名称的唯一性
        validateArticleCatogryParentNameUnique(null, createReqVO.getParentId(), createReqVO.getParentName());

        // 插入
        ArticleCatogryDO articleCatogry = BeanUtils.toBean(createReqVO, ArticleCatogryDO.class);
        articleCatogryMapper.insert(articleCatogry);
        // 返回
        return articleCatogry.getId();
    }

    @Override
    public void updateArticleCatogry(ArticleCatogrySaveReqVO updateReqVO) {
        // 校验存在
        validateArticleCatogryExists(updateReqVO.getId());
        // 校验父ID的有效性
        validateParentArticleCatogry(updateReqVO.getId(), updateReqVO.getParentId());
        // 校验父分类名称的唯一性
        validateArticleCatogryParentNameUnique(updateReqVO.getId(), updateReqVO.getParentId(), updateReqVO.getParentName());

        // 更新
        ArticleCatogryDO updateObj = BeanUtils.toBean(updateReqVO, ArticleCatogryDO.class);
        articleCatogryMapper.updateById(updateObj);
    }

    @Override
    public void deleteArticleCatogry(Long id) {
        // 校验存在
        validateArticleCatogryExists(id);
        // 校验是否有子文章分类
        if (articleCatogryMapper.selectCountByParentId(id) > 0) {
            throw exception(ARTICLE_CATOGRY_EXITS_CHILDREN);
        }
        // 删除
        articleCatogryMapper.deleteById(id);
    }

    private void validateArticleCatogryExists(Long id) {
        if (articleCatogryMapper.selectById(id) == null) {
            throw exception(ARTICLE_CATOGRY_NOT_EXISTS);
        }
    }

    private void validateParentArticleCatogry(Long id, Long parentId) {
        if (parentId == null || ArticleCatogryDO.PARENT_ID_ROOT.equals(parentId)) {
            return;
        }
        // 1. 不能设置自己为父文章分类
        if (Objects.equals(id, parentId)) {
            throw exception(ARTICLE_CATOGRY_PARENT_ERROR);
        }
        // 2. 父文章分类不存在
        ArticleCatogryDO parentArticleCatogry = articleCatogryMapper.selectById(parentId);
        if (parentArticleCatogry == null) {
            throw exception(ARTICLE_CATOGRY_PARENT_NOT_EXITS);
        }
        // 3. 递归校验父文章分类，如果父文章分类是自己的子文章分类，则报错，避免形成环路
        if (id == null) { // id 为空，说明新增，不需要考虑环路
            return;
        }
        for (int i = 0; i < Short.MAX_VALUE; i++) {
            // 3.1 校验环路
            parentId = parentArticleCatogry.getParentId();
            if (Objects.equals(id, parentId)) {
                throw exception(ARTICLE_CATOGRY_PARENT_IS_CHILD);
            }
            // 3.2 继续递归下一级父文章分类
            if (parentId == null || ArticleCatogryDO.PARENT_ID_ROOT.equals(parentId)) {
                break;
            }
            parentArticleCatogry = articleCatogryMapper.selectById(parentId);
            if (parentArticleCatogry == null) {
                break;
            }
        }
    }

    private void validateArticleCatogryParentNameUnique(Long id, Long parentId, String parentName) {
        ArticleCatogryDO articleCatogry = articleCatogryMapper.selectByParentIdAndParentName(parentId, parentName);
        if (articleCatogry == null) {
            return;
        }
        // 如果 id 为空，说明不用比较是否为相同 id 的文章分类
        if (id == null) {
            throw exception(ARTICLE_CATOGRY_PARENT_NAME_DUPLICATE);
        }
        if (!Objects.equals(articleCatogry.getId(), id)) {
            throw exception(ARTICLE_CATOGRY_PARENT_NAME_DUPLICATE);
        }
    }

    @Override
    public ArticleCatogryDO getArticleCatogry(Long id) {
        return articleCatogryMapper.selectById(id);
    }

    @Override
    public List<ArticleCatogryDO> getArticleCatogryList(ArticleCatogryListReqVO listReqVO) {
        return articleCatogryMapper.selectList(listReqVO);
    }

}
